UNISWAP_ETH_INFO = {
    'factoryAddress': '0x5c69bee701ef814a2b6a3edd4b1652cb9cc5aa6f',
}

UNISWAP_V3_ETH_INFO = {
    'factoryAddress': '0x1F98431c8aD98523631AE4a59f267346ea31F984',
    'staker_address': '0xe34139463bA50bD61336E0c446Bd8C0867c6fE65',
    'NFT_manager_address': '0xC36442b4a4522E871399CD717aBDD847Ab11FE88',
}
